<?php
/**
 * FSF Member Dashboard
 * Copyright © 2014 Free Software Foundation, Inc.
 *
 * This file is a part of FSF Member Dashboard.
 *
 * FSF Member Dashboard is free software; you can copy, modify, and
 * distribute it under the terms of the GNU Affero General Public
 * License Version 3, 19 November 2007 and the CiviCRM Licensing
 * Exception.
 *
 * FSF Member Dashboard is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FSF Member Dashboard.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

require_once 'CRM/Memberdashboard/Page.php';

class CRM_Memberdashboard_Page_Benefits extends CRM_Memberdashboard_Page {
  function loadEmailAliases() {
    return CRM_Memberdashboard_BAO_EmailAlias::allForContact($this->contact['id']);
  }

  function hasValidMembership() {
    // Since we don't currently shut off an expired member's benefits,
    // we only test that they have a membership, not that it is current.
    $params = array(
      'contact_id' => $this->contact['id']
    );

    try {
      $result = civicrm_api3('membership', 'get', $params);
      return $result['count'] > 0;
    } catch (CiviCRM_API3_Exception $e) {
      return false;
    }
  }

  function postProcess() {
    $action = $_REQUEST['action'];

    // Short-circuit if user doesn't have a membership.
    if($this->hasValidMembership()) {
      switch($action) {
        case 'add':
          $this->add();
          break;

        case 'edit':
          $this->edit();
          break;
      }
    } else {
      CRM_Core_Session::setStatus('Only members can have email aliases',
                                  'Error', 'error');
    }
  }

  function add() {
    $contactId = $this->contact['id'];
    $localpart = $_POST['localpart'];
    $forward = $_POST['forward'];
    $count = CRM_Memberdashboard_BAO_EmailAlias::countForContact($contactId);

    if(CRM_Utils_Rule::email($forward)) {
      if(CRM_Utils_Rule::email("$localpart@example.com")) {
        if($count < MEMBERDASHBOARD_MAX_EMAIL_ALIASES) {
          try {
            $params = array(
              'contact_id' => $contactId,
              'localpart' => $localpart,
              'forward' => $forward
            );
            CRM_Memberdashboard_BAO_EmailAlias::create($params);
            CRM_Core_Session::setStatus('Email alias added!',
                                        'Success', 'success');
          } catch (Exception $e) {
            CRM_Core_Session::setStatus('Failed to create email alias',
                                        'Error', 'error');
          }
        } else {
          CRM_Core_Session::setStatus('You cannot have more than 5 email aliases',
                                      'Error', 'error');
        }
      } else {
        CRM_Core_Session::setStatus('Invalid local part',
                                    'Error', 'error');
      }
    } else {
      CRM_Core_Session::setStatus('Invalid email forwarding address',
                                  'Error', 'error');
    }
  }

  function edit() {
    $ids = $_POST['ids'];
    $localparts = $_POST['localparts'];
    $forwards = $_POST['forwards'];
    $delete = isset($_POST['delete']) ? $_POST['delete'] : array();
    $aliases = CRM_Memberdashboard_BAO_EmailAlias::find($ids);
    $error = '';

    for($i = 0; $i < count($ids); $i++) {
      $id = $ids[$i];
      $localpart = $localparts[$i];
      $forward = $forwards[$i];

      if(isset($delete[$id]) && $delete[$id] == 'delete') {
        CRM_Memberdashboard_BAO_EmailAlias::destroy($id);
      } else {
        $alias = $aliases[$id];

        // Make sure sneaky users can't edit other people's aliases.
        if($alias->contactId != $this->contact['id']) {
          $error .= '<li>You cannot alter email aliases that don\'t belong to you!</li>';
        } else if(CRM_Utils_Rule::email($forward)) {
          if(CRM_Utils_Rule::email("$localpart@example.com")) {
            $alias->localpart = $localpart;
            $alias->forward = $forward;
            $alias->save();
          } else {
            $error .= "<li>Invalid local part: $localpart</li>";
          }
        } else {
          $error .= "<li>Invalid email forwarding address: $forward</li>";
        }
      }
    }

    if(empty($error)) {
      CRM_Core_Session::setStatus("Email aliases updated!", 'Success', 'success');
    } else {
      CRM_Core_Session::setStatus("<ul>$error</ul>", 'Error', 'error');
    }
  }

  function run() {
    // FIXME: Assumes CiviCRM is running on Drupal.
    // Ugly global variable for the current Drupal user.
    global $user;

    // create/update/delete email aliases.
    if($this->isPOST()) {
      $this->postProcess();
    }

    $emailAliases = $this->loadEmailAliases();

    CRM_Utils_System::setTitle(ts('Benefits'));

    $this->assign('user', $user);
    $this->assign('emailAliases', $emailAliases);
    $this->assign('hasValidMembership', $this->hasValidMembership());

    parent::run();
  }
}
